/* $Id: i2c-winbond.c,v 1.0 26th Feb 2002 $
 * i2c-winbond.c - driver for i2c winbond inside Super I/O
 *
 * Copyright (C) 2002 Lineo (Embedix, Inc.)
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/fs.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/miscdevice.h>

#include <asm/io.h>
#include <asm/oplib.h>
#include <asm/uaccess.h>

#include <asm/mostek.h>
#include <linux/winbond.h>

#define TRUE		1
#define FALSE 		0
#define I2C_ADDR	0x1fe02004000LL
#define STICK_LO_ADDR	0x1fe0000f070LL
#define STICK_HI_ADDR	0x1fe0000f078LL
#define WINB_ADDR	0x5a
#define lduba_15(addr)	mostek_read(addr)
#define stba_15(x,addr)	mostek_write(addr,x)

MODULE_AUTHOR(Lineo - Embedix Inc. <www.lineo.com>);
MODULE_DESCRIPTION("Hardware i2c winbond driver for Sun ");
MODULE_SUPPORTED_DEVICE("i2c-winbond");

#define RIOWD_NAME	"i2c"
#define RIOWD_MINOR	216


static int i2c_open(struct inode *inode, struct file *filp)
{
	return 0;
}

static int i2c_release(struct inode *inode, struct file *filp)
{
	return 0;
}

static int i2c_ioctl(struct inode *inode, struct file *filp,
		       unsigned int cmd, unsigned long arg)
{
	unsigned char data, dataf;
	unsigned short datas;
	int fanspeed;
	unsigned char setpwm;

	switch (cmd) {
	case WINBOND_GET_TEMP0:
		data = i2c_get_byte (WINB_ADDR, 0x27);	
		copy_to_user((unsigned char*)arg,(unsigned char*)&data,sizeof(unsigned char));
		break;

	case WINBOND_GET_TEMP1:
		data = i2c_get_byte (WINB_ADDR, 0x4e);	
		data &= ~0x07;
		data |= 0x01;
		i2c_set_byte (WINB_ADDR, 0x4e, data);
		datas = i2c_get_byte (WINB_ADDR, 0x51);//low byte	
		data = i2c_get_byte (WINB_ADDR, 0x4e);	
		data &= ~0x07;
		data |= 0x01;
		i2c_set_byte (WINB_ADDR, 0x4e, data);
		datas |= (8 << i2c_get_byte (WINB_ADDR, 0x50));//high byte	
		datas &= 0x01ff;
		copy_to_user((unsigned short *)arg,(unsigned short *)&datas,sizeof(unsigned short));
		break;

	case WINBOND_GET_TEMP2:
		data = i2c_get_byte (WINB_ADDR, 0x4e);	
		data &= ~0x07;
		data |= 0x02;
		i2c_set_byte (WINB_ADDR, 0x4e, data);
		datas = i2c_get_byte (WINB_ADDR, 0x51);//low byte	
		data = i2c_get_byte (WINB_ADDR, 0x4e);	
		data &= ~0x07;
		data |= 0x02;
		i2c_set_byte (WINB_ADDR, 0x4e, data);
		datas |= (8 << (i2c_get_byte (WINB_ADDR, 0x50)));//high byte	
		datas &= 0x01ff;
		copy_to_user((unsigned short *)arg,(unsigned short *)&datas,sizeof(unsigned short));
		break;
		
	case WINBOND_SET_FAN0SPEED:
		copy_from_user((unsigned char*)&dataf,(unsigned char*)arg,sizeof(unsigned char));  
		//Enable the output control
		data = i2c_get_byte (WINB_ADDR, 0x4d);	
		data &= ~0x03;
		data |= 0x02;
		i2c_set_byte (WINB_ADDR, 0x4d, data);

		//Set Bank 0
		data = i2c_get_byte (WINB_ADDR, 0x4e);	
		data &= ~0x07;
		i2c_set_byte (WINB_ADDR, 0x4e, data);
		
		i2c_set_byte (WINB_ADDR, 0x5b, dataf); //set PWMOUT1
		
		//Disable the output control
		data = i2c_get_byte (WINB_ADDR, 0x4d);	
		data &= ~0x03;
		data |= 0x01;
		i2c_set_byte (WINB_ADDR, 0x4d, data);
		break;

	case WINBOND_GET_FAN0:
		data = i2c_get_byte (WINB_ADDR, 0x28);	
		copy_to_user((unsigned char*)arg,(unsigned char*)&data,sizeof(unsigned char));
		break;
		
	case WINBOND_SET_FAN1SPEED:
		return -EINVAL;

	case WINBOND_GET_FAN1:
		return -EINVAL;
		
	case WINBOND_SET_FAN2SPEED:
		return -EINVAL;
		break;

	case WINBOND_GET_FAN2:
		return -EINVAL;
		
	case WINBOND_GET_VCOREA:
		data = i2c_get_byte (WINB_ADDR, 0x20);	
		copy_to_user((unsigned char*)arg,(unsigned char*)&data,sizeof(unsigned char));
		break;
		
	case WINBOND_GET_VSB:
		data = i2c_get_byte (WINB_ADDR, 0x4e);	
		data &= ~0x07;
		data |= 0x05;
		i2c_set_byte (WINB_ADDR, 0x4e, data);
		data = i2c_get_byte (WINB_ADDR, 0x50);	
		copy_to_user((unsigned char*)arg,(unsigned char*)&data,sizeof(unsigned char));
		break;

	case WINBOND_GET_PLUS3:
		data = i2c_get_byte (WINB_ADDR, 0x22);	
		copy_to_user((unsigned char*)arg,(unsigned char*)&data,sizeof(unsigned char));
		break;

	case WINBOND_GET_PLUS5:
		data = i2c_get_byte (WINB_ADDR, 0x23);	
		copy_to_user((unsigned char*)arg,(unsigned char*)&data,sizeof(unsigned char));
		break;
		
	case WINBOND_GET_PLUS12:
		data = i2c_get_byte (WINB_ADDR, 0x24);	
		copy_to_user((unsigned char*)arg,(unsigned char*)&data,sizeof(unsigned char));
		break;

	case WINBOND_GET_MINUS12:
		data = i2c_get_byte (WINB_ADDR, 0x25);	
		copy_to_user((unsigned char*)arg,(unsigned char*)&data,sizeof(unsigned char));
		break;

	case WINBOND_GET_MINUS5:
		data = i2c_get_byte (WINB_ADDR, 0x26);	
		copy_to_user((unsigned char*)arg,(unsigned char*)&data,sizeof(unsigned char));
		break;
		
	default:
		return -EINVAL;
	};

	return 0;
}

static int i2c_write(struct file *file, const char *buf, int count)
{
	return 0;
}

static ssize_t i2c_read(struct file *file, char *buffer, size_t count, loff_t *ppos)
{
	return 0;
}

static struct file_operations i2c_fops = {
	owner:		THIS_MODULE,
	ioctl:		i2c_ioctl,
	open:		i2c_open,
	write:		i2c_write,
	read:		i2c_read,
	release:	i2c_release,
};

static struct miscdevice i2c_miscdev = { RIOWD_MINOR, RIOWD_NAME, &i2c_fops };

static int __init i2c_init(void)
{
	u_char status, time_in_min;

	winbond_setup();
	if (misc_register(&i2c_miscdev)) {
		printk(KERN_ERR "i2c-winbond: Cannot register i2c-winbond misc device.\n");
		return -1;
	}
	return 0;
}

static void __exit i2c_cleanup(void)
{
	misc_deregister(&i2c_miscdev);
}

/* Additional code for i2c of Southbridge */

void
winbond_setup (void)
{
	u_char data;

	i2c_set_byte (WINB_ADDR, 0x4d, 0x95);		// Set CR[4d] bit 7 = 1

	data = i2c_get_byte (WINB_ADDR, 0x4b);		// Set CR[4b] bits [2-3]
	data = ((data & (~0xc)) | 8);				//   Clock input 48MHz
	i2c_set_byte (WINB_ADDR, 0x4b, data);		//

	data = i2c_get_byte (WINB_ADDR, 0x5d);		// Set CR[5d] bit 2
	data |= 4;									//   Sensor 2 is diode
	i2c_set_byte (WINB_ADDR, 0x5d, data);		//
}

void
i2c_delay (void)
{
	unsigned long i = 0;
	for (i = 0;i < 0x2fffff;i++);
}


int
i2c_get_byte (u_char device, u_char byte)
{
	int i, count=10;
	u_char temp, data;

try_again:
	if (count != 10)
	 	printk ("No RSP from I2C device, retrying\n\r", 0, 0, 0, 0);
	count--;
    	if (count == 0)
		return (TRUE);

	stba_15 (0x0c, (I2C_ADDR+1));
	stba_15 (0xff, (I2C_ADDR));

	i2c_delay ();

	for (i=1000; (((lduba_15 ((I2C_ADDR)) & 4) == 0) && (i>0)); i--);
	if (i==0) return (TRUE);

	stba_15 ((device | 1), (I2C_ADDR+3));
	stba_15 (0x20, (I2C_ADDR+1));
	stba_15 (byte, (I2C_ADDR+7));

	for (i=1000; (((lduba_15 ((I2C_ADDR)) & 4) == 0) && (i>0)); i--);
	if (i==0) return (TRUE);

	stba_15 (0xff, (I2C_ADDR+2));

	i2c_delay ();

	if (lduba_15 ((I2C_ADDR)) & 0x40)	// No ACK?
		return (TRUE);

	for (i=100; i>0; i--)
	{
		temp = lduba_15 ((I2C_ADDR));

		if (temp == 0x4)		// I2C idle
			goto try_again;

		if (temp == 0x44)		// No ACK
			return (TRUE);

		if (temp & 0x20)		// SMB bus timeout
			goto try_again;

		if (temp & 0x14)		// Valid response
			break;
	}

	if (i==0) return (TRUE);

	data = lduba_15 ((I2C_ADDR+4));
	return (data);
}

int
i2c_set_byte (u_char device, u_char offset, u_char data)
{
	int i, count=10;
	u_char temp;

try_again1:
	if (count != 10)
		printk ("No RSP from I2C device, retrying\n");
	count--;
    if (count == 0)
		return (TRUE);

	//mostek_write((I2C_ADDR+1),0x0c);
	stba_15 (0x0c, (I2C_ADDR+1));
	stba_15 (0xff, (I2C_ADDR));

	i2c_delay ();

	for (i=1000; (((lduba_15 ((I2C_ADDR)) & 4) == 0) && (i>0)); i--);
	if (i==0) return (TRUE);

	stba_15 (device,(I2C_ADDR+3));
	stba_15 (0x20, (I2C_ADDR+1));
	stba_15 (offset, (I2C_ADDR+7));
	stba_15 (data, (I2C_ADDR+4));

	for (i=1000; (((lduba_15 ((u_char *) (I2C_ADDR)) & 4) == 0) && (i>0)); i--);
	if (i==0) return (TRUE);

	stba_15 (0xff, (I2C_ADDR+2));

	i2c_delay ();

	if (lduba_15 ((I2C_ADDR)) & 0x40)	// No ACK?
		return (TRUE);

	for (i=100; i>0; i--)
	{
		temp = lduba_15 ((I2C_ADDR));

		if (temp == 0x4)		// I2C idle
			goto try_again1;

		if (temp == 0x44)		// No ACK
			return (TRUE);

		if (temp & 0x20)		// SMB bus timeout
			goto try_again1;

		if (temp & 0x14)		// Valid response
			break;
	}

	if (i==0) return (TRUE);

	return (FALSE);
}

module_init(i2c_init);
module_exit(i2c_cleanup);



